@@ -10,6 +10,7 @@
 #include "flang/Common/idioms.h"
 #include "llvm/Support/raw_ostream.h"
 #include <algorithm>
+#include <set>
 #include <utility>
 
 namespace Fortran::parser {
@@ -251,15 +252,14 @@ void AllSources::EmitMessage(llvm::raw_ostream &o,
   common::visit(
       common::visitors{
           [&](const Inclusion &inc) {
-            o << inc.source.path();
             std::size_t offset{origin.covers.MemberOffset(range->start())};
-            SourcePosition pos{inc.source.FindOffsetLineAndColumn(offset)};
-            o << ':' << pos.line << ':' << pos.column << ": ";
+            SourcePosition pos{inc.source.GetSourcePosition(offset)};
+            o << pos.path << ':' << pos.line << ':' << pos.column << ": ";
             EmitPrefix(o, color, prefix, this->getShowColors());
             o << message << '\n';
             if (echoSourceLine) {
               const char *text{inc.source.content().data() +
-                  inc.source.GetLineStartOffset(pos.line)};
+                  inc.source.GetLineStartOffset(pos.trueLineNumber)};
               o << "  ";
               for (const char *p{text}; *p != '\n'; ++p) {
                 o << *p;
@@ -274,7 +274,7 @@ void AllSources::EmitMessage(llvm::raw_ostream &o,
                 auto last{range->start() + range->size() - 1};
                 if (&MapToOrigin(last) == &origin) {
                   auto endOffset{origin.covers.MemberOffset(last)};
-                  auto endPos{inc.source.FindOffsetLineAndColumn(endOffset)};
+                  auto endPos{inc.source.GetSourcePosition(endOffset)};
                   if (pos.line == endPos.line) {
                     for (int j{pos.column}; j < endPos.column; ++j) {
                       o << '^';
@@ -293,7 +293,7 @@ void AllSources::EmitMessage(llvm::raw_ostream &o,
           [&](const Macro &mac) {
             EmitMessage(
                 o, origin.replaces, message, prefix, color, echoSourceLine);
-            EmitMessage(o, mac.definition, "in a macro defined here", prefix,
+            EmitMessage(o, mac.definition, "in a macro defined here", ""s,
                 color, echoSourceLine);
             if (echoSourceLine) {
               o << "that expanded to:\n  " << mac.expansion << "\n  ";
@@ -351,7 +351,7 @@ std::optional<SourcePosition> AllSources::GetSourcePosition(
       common::visitors{
           [&](const Inclusion &inc) -> std::optional<SourcePosition> {
             std::size_t offset{origin.covers.MemberOffset(prov)};
-            return inc.source.FindOffsetLineAndColumn(offset);
+            return inc.source.GetSourcePosition(offset);
           },
           [&](const Macro &) {
             return GetSourcePosition(origin.replaces.start());
@@ -373,14 +373,15 @@ std::optional<ProvenanceRange> AllSources::GetFirstFileProvenance() const {
 }
 
 std::string AllSources::GetPath(Provenance at) const {
-  const SourceFile *source{GetSourceFile(at)};
-  return source ? source->path() : ""s;
+  std::size_t offset{0};
+  const SourceFile *source{GetSourceFile(at, &offset)};
+  return source ? *source->GetSourcePosition(offset).path : ""s;
 }
 
 int AllSources::GetLineNumber(Provenance at) const {
   std::size_t offset{0};
   const SourceFile *source{GetSourceFile(at, &offset)};
-  return source ? source->FindOffsetLineAndColumn(offset).line : 0;
+  return source ? source->GetSourcePosition(offset).line : 0;
 }
 
 Provenance AllSources::CompilerInsertionProvenance(char ch) {
@@ -451,6 +452,14 @@ const AllSources::Origin &AllSources::MapToOrigin(Provenance at) const {
   return origin_[low];
 }
 
+Provenance AllSources::GetReplacedProvenance(Provenance provenance) const {
+  const Origin &origin{MapToOrigin(provenance)};
+  if (std::holds_alternative<Macro>(origin.u)) {
+    return origin.replaces.start();
+  }
+  return provenance;
+}
+
 std::optional<ProvenanceRange> CookedSource::GetProvenanceRange(
     CharBlock cookedRange) const {
   if (!AsCharBlock().Contains(cookedRange)) {
@@ -464,7 +473,16 @@ std::optional<ProvenanceRange> CookedSource::GetProvenanceRange(
   if (first.start() <= last.start()) {
     return {ProvenanceRange{first.start(), last.start() - first.start() + 1}};
   } else {
-    return std::nullopt;
+    // cookedRange may start (resp. end) in a macro expansion while it does not
+    // end (resp. start) in this macro expansion. Attempt to build a range
+    // over the replaced source.
+    Provenance firstStart{allSources_.GetReplacedProvenance(first.start())};
+    Provenance lastStart{allSources_.GetReplacedProvenance(last.start())};
+    if (firstStart <= lastStart) {
+      return {ProvenanceRange{firstStart, lastStart - firstStart + 1}};
+    } else {
+      return std::nullopt;
+    }
   }
 }
 
@@ -529,6 +547,7 @@ llvm::raw_ostream &AllSources::Dump(llvm::raw_ostream &o) const {
   o << "AllSources range_ ";
   DumpRange(o, range_);
   o << '\n';
+  std::set<const SourceFile *> sources;
   for (const Origin &m : origin_) {
     o << "   ";
     DumpRange(o, m.covers);
@@ -539,6 +558,7 @@ llvm::raw_ostream &AllSources::Dump(llvm::raw_ostream &o) const {
                           o << "module ";
                         }
                         o << "file " << inc.source.path();
+                        sources.emplace(&inc.source);
                       },
                       [&](const Macro &mac) { o << "macro " << mac.expansion; },
                       [&](const CompilerInsertion &ins) {
@@ -557,6 +577,9 @@ llvm::raw_ostream &AllSources::Dump(llvm::raw_ostream &o) const {
     }
     o << '\n';
   }
+  for (const SourceFile *sf : sources) {
+    sf->Dump(o);
+  }
   return o;
 }
 
@@ -572,7 +595,7 @@ AllCookedSources::AllCookedSources(AllSources &s) : allSources_{s} {}
 AllCookedSources::~AllCookedSources() {}
 
 CookedSource &AllCookedSources::NewCookedSource() {
-  return cooked_.emplace_back();
+  return cooked_.emplace_back(allSources_);
 }
 
 const CookedSource *AllCookedSources::Find(CharBlock x) const {
