@@ -9,10 +9,10 @@
 #ifndef LLVM_LIBC_SRC_MATH_NEXTAFTERF_H
 #define LLVM_LIBC_SRC_MATH_NEXTAFTERF_H
 
-namespace __llvm_libc {
+namespace LIBC_NAMESPACE {
 
 float nextafterf(float x, float y);
 
-} // namespace __llvm_libc
+} // namespace LIBC_NAMESPACE
 
 #endif // LLVM_LIBC_SRC_MATH_NEXTAFTERF_H
