@@ -10,24 +10,26 @@
 
 #include "src/__support/FPUtil/FPBits.h"
 #include "src/math/fdimf.h"
-#include "utils/UnitTest/FPMatcher.h"
-#include "utils/UnitTest/Test.h"
+#include "test/UnitTest/FPMatcher.h"
+#include "test/UnitTest/Test.h"
 #include <math.h>
 
 using LlvmLibcFDimTest = FDimTestTemplate<float>;
 
-TEST_F(LlvmLibcFDimTest, NaNArg_fdimf) { test_na_n_arg(&__llvm_libc::fdimf); }
+TEST_F(LlvmLibcFDimTest, NaNArg_fdimf) {
+  test_na_n_arg(&LIBC_NAMESPACE::fdimf);
+}
 
-TEST_F(LlvmLibcFDimTest, InfArg_fdimf) { test_inf_arg(&__llvm_libc::fdimf); }
+TEST_F(LlvmLibcFDimTest, InfArg_fdimf) { test_inf_arg(&LIBC_NAMESPACE::fdimf); }
 
 TEST_F(LlvmLibcFDimTest, NegInfArg_fdimf) {
-  test_neg_inf_arg(&__llvm_libc::fdimf);
+  test_neg_inf_arg(&LIBC_NAMESPACE::fdimf);
 }
 
 TEST_F(LlvmLibcFDimTest, BothZero_fdimf) {
-  test_both_zero(&__llvm_libc::fdimf);
+  test_both_zero(&LIBC_NAMESPACE::fdimf);
 }
 
 TEST_F(LlvmLibcFDimTest, InFloatRange_fdimf) {
-  test_in_range(&__llvm_libc::fdimf);
+  test_in_range(&LIBC_NAMESPACE::fdimf);
 }
