@@ -94,6 +94,8 @@ constexpr bool test_all() {
   using std::ranges::swap_ranges_result;
   using std::ranges::unary_transform_result;
   using std::ranges::unique_copy_result;
+  using InIter = std::array<int, 3>::iterator;
+  using OutIter = std::array<int, 6>::iterator;
 
   auto unary_pred = [](int i) { return i > 0; };
   auto binary_pred = [](int i, int j) { return i < j; };
@@ -102,7 +104,7 @@ constexpr bool test_all() {
   std::array in = {1, 2, 3};
   std::array in2 = {4, 5, 6};
 
-  auto mid = in.begin() + 1;
+  auto mid = in.data() + 1;
 
   std::array output = {7, 8, 9, 10, 11, 12};
   auto out = output.begin();
@@ -116,8 +118,8 @@ constexpr bool test_all() {
   dangling_1st(std::ranges::find_if_not, in, unary_pred);
   dangling_1st(std::ranges::find_first_of, in, in2);
   dangling_1st(std::ranges::adjacent_find, in);
-  dangling_1st<mismatch_result<dangling, int*>>(std::ranges::mismatch, in, in2);
-  dangling_2nd<mismatch_result<int*, dangling>>(std::ranges::mismatch, in, in2);
+  dangling_1st<mismatch_result<dangling, InIter>>(std::ranges::mismatch, in, in2);
+  dangling_2nd<mismatch_result<InIter, dangling>>(std::ranges::mismatch, in, in2);
   dangling_both<mismatch_result<dangling, dangling>>(std::ranges::mismatch, in, in2);
   dangling_1st(std::ranges::partition_point, in, unary_pred);
   dangling_1st(std::ranges::lower_bound, in, x);
@@ -132,55 +134,56 @@ constexpr bool test_all() {
   dangling_1st(std::ranges::is_sorted_until, in);
   dangling_1st(std::ranges::is_heap_until, in);
   dangling_1st<for_each_result<dangling, decltype(unary_pred)>>(std::ranges::for_each, in, unary_pred);
-  dangling_1st<copy_result<dangling, int*>>(std::ranges::copy, in, out);
-  dangling_1st<copy_backward_result<dangling, int*>>(std::ranges::copy_backward, in, output.end());
-  dangling_1st<copy_if_result<dangling, int*>>(std::ranges::copy_if, in, out, unary_pred);
-  dangling_1st<move_result<dangling, int*>>(std::ranges::move, in, out);
-  dangling_1st<move_backward_result<dangling, int*>>(std::ranges::move_backward, in, output.end());
+  dangling_1st<copy_result<dangling, OutIter>>(std::ranges::copy, in, out);
+  dangling_1st<copy_backward_result<dangling, OutIter>>(std::ranges::copy_backward, in, output.end());
+  dangling_1st<copy_if_result<dangling, OutIter>>(std::ranges::copy_if, in, out, unary_pred);
+  dangling_1st<move_result<dangling, OutIter>>(std::ranges::move, in, out);
+  dangling_1st<move_backward_result<dangling, OutIter>>(std::ranges::move_backward, in, output.end());
   dangling_1st(std::ranges::fill, in, x);
   { // transform
+    using OutTransformIter = std::array<bool, 3>::iterator;
     std::array out_transform = {false, true, true};
-    dangling_1st<unary_transform_result<dangling, bool*>>(std::ranges::transform, in, out_transform.begin(), unary_pred);
-    dangling_1st<binary_transform_result<dangling, int*, bool*>>(
+    dangling_1st<unary_transform_result<dangling, OutTransformIter>>(std::ranges::transform, in, out_transform.begin(), unary_pred);
+    dangling_1st<binary_transform_result<dangling, InIter, OutTransformIter>>(
         std::ranges::transform, in, in2, out_transform.begin(), binary_pred);
-    dangling_2nd<binary_transform_result<int*, dangling, bool*>>(
+    dangling_2nd<binary_transform_result<InIter, dangling, OutTransformIter>>(
         std::ranges::transform, in, in2, out_transform.begin(), binary_pred);
-    dangling_both<binary_transform_result<dangling, dangling, bool*>>(
+    dangling_both<binary_transform_result<dangling, dangling, OutTransformIter>>(
         std::ranges::transform, in, in2, out_transform.begin(), binary_pred);
   }
   dangling_1st(std::ranges::generate, in, gen);
-  dangling_1st<remove_copy_result<dangling, int*>>(std::ranges::remove_copy, in, out, x);
-  dangling_1st<remove_copy_if_result<dangling, int*>>(std::ranges::remove_copy_if, in, out, unary_pred);
+  dangling_1st<remove_copy_result<dangling, OutIter>>(std::ranges::remove_copy, in, out, x);
+  dangling_1st<remove_copy_if_result<dangling, OutIter>>(std::ranges::remove_copy_if, in, out, unary_pred);
   dangling_1st(std::ranges::replace, in, x, x);
   dangling_1st(std::ranges::replace_if, in, std::identity{}, x);
-  dangling_1st<replace_copy_result<dangling, int*>>(std::ranges::replace_copy, in, out, x, x);
-  dangling_1st<replace_copy_if_result<dangling, int*>>(std::ranges::replace_copy_if, in, out, unary_pred, x);
-  dangling_1st<swap_ranges_result<dangling, int*>>(std::ranges::swap_ranges, in, in2);
-  dangling_2nd<swap_ranges_result<int*, dangling>>(std::ranges::swap_ranges, in, in2);
+  dangling_1st<replace_copy_result<dangling, OutIter>>(std::ranges::replace_copy, in, out, x, x);
+  dangling_1st<replace_copy_if_result<dangling, OutIter>>(std::ranges::replace_copy_if, in, out, unary_pred, x);
+  dangling_1st<swap_ranges_result<dangling, InIter>>(std::ranges::swap_ranges, in, in2);
+  dangling_2nd<swap_ranges_result<InIter, dangling>>(std::ranges::swap_ranges, in, in2);
   dangling_both<swap_ranges_result<dangling, dangling>>(std::ranges::swap_ranges, in, in2);
-  dangling_1st<reverse_copy_result<dangling, int*>>(std::ranges::reverse_copy, in, out);
-  dangling_1st<rotate_copy_result<dangling, int*>>(std::ranges::rotate_copy, in, mid, out);
-  dangling_1st<unique_copy_result<dangling, int*>>(std::ranges::unique_copy, in, out);
-  dangling_1st<partition_copy_result<dangling, int*, int*>>(std::ranges::partition_copy, in, out, out2, unary_pred);
-  dangling_1st<partial_sort_copy_result<dangling, int*>>(std::ranges::partial_sort_copy, in, in2);
-  dangling_2nd<partial_sort_copy_result<int*, dangling>>(std::ranges::partial_sort_copy, in, in2);
+  dangling_1st<reverse_copy_result<dangling, OutIter>>(std::ranges::reverse_copy, in, out);
+  dangling_1st<rotate_copy_result<dangling, OutIter>>(std::ranges::rotate_copy, in, mid, out);
+  dangling_1st<unique_copy_result<dangling, OutIter>>(std::ranges::unique_copy, in, out);
+  dangling_1st<partition_copy_result<dangling, OutIter, OutIter>>(std::ranges::partition_copy, in, out, out2, unary_pred);
+  dangling_1st<partial_sort_copy_result<dangling, InIter>>(std::ranges::partial_sort_copy, in, in2);
+  dangling_2nd<partial_sort_copy_result<InIter, dangling>>(std::ranges::partial_sort_copy, in, in2);
   dangling_both<partial_sort_copy_result<dangling, dangling>>(std::ranges::partial_sort_copy, in, in2);
-  dangling_1st<merge_result<dangling, int*, int*>>(std::ranges::merge, in, in2, out);
-  dangling_2nd<merge_result<int*, dangling, int*>>(std::ranges::merge, in, in2, out);
-  dangling_both<merge_result<dangling, dangling, int*>>(std::ranges::merge, in, in2, out);
-  dangling_1st<set_difference_result<dangling, int*>>(std::ranges::set_difference, in, in2, out);
-  dangling_1st<set_intersection_result<dangling, int*, int*>>(std::ranges::set_intersection, in, in2, out);
-  dangling_2nd<set_intersection_result<int*, dangling, int*>>(std::ranges::set_intersection, in, in2, out);
-  dangling_both<set_intersection_result<dangling, dangling, int*>>(std::ranges::set_intersection, in, in2, out);
-  dangling_1st<set_symmetric_difference_result<dangling, int*, int*>>(
+  dangling_1st<merge_result<dangling, InIter, OutIter>>(std::ranges::merge, in, in2, out);
+  dangling_2nd<merge_result<InIter, dangling, OutIter>>(std::ranges::merge, in, in2, out);
+  dangling_both<merge_result<dangling, dangling, OutIter>>(std::ranges::merge, in, in2, out);
+  dangling_1st<set_difference_result<dangling, OutIter>>(std::ranges::set_difference, in, in2, out);
+  dangling_1st<set_intersection_result<dangling, InIter, OutIter>>(std::ranges::set_intersection, in, in2, out);
+  dangling_2nd<set_intersection_result<InIter, dangling, OutIter>>(std::ranges::set_intersection, in, in2, out);
+  dangling_both<set_intersection_result<dangling, dangling, OutIter>>(std::ranges::set_intersection, in, in2, out);
+  dangling_1st<set_symmetric_difference_result<dangling, InIter, OutIter>>(
       std::ranges::set_symmetric_difference, in, in2, out);
-  dangling_2nd<set_symmetric_difference_result<int*, dangling, int*>>(
+  dangling_2nd<set_symmetric_difference_result<InIter, dangling, OutIter>>(
       std::ranges::set_symmetric_difference, in, in2, out);
-  dangling_both<set_symmetric_difference_result<dangling, dangling, int*>>(
+  dangling_both<set_symmetric_difference_result<dangling, dangling, OutIter>>(
       std::ranges::set_symmetric_difference, in, in2, out);
-  dangling_1st<set_union_result<dangling, int*, int*>>(std::ranges::set_union, in, in2, out);
-  dangling_2nd<set_union_result<int*, dangling, int*>>(std::ranges::set_union, in, in2, out);
-  dangling_both<set_union_result<dangling, dangling, int*>>(std::ranges::set_union, in, in2, out);
+  dangling_1st<set_union_result<dangling, InIter, OutIter>>(std::ranges::set_union, in, in2, out);
+  dangling_2nd<set_union_result<InIter, dangling, OutIter>>(std::ranges::set_union, in, in2, out);
+  dangling_both<set_union_result<dangling, dangling, OutIter>>(std::ranges::set_union, in, in2, out);
   dangling_1st(std::ranges::remove, in, x);
   dangling_1st(std::ranges::remove_if, in, unary_pred);
   dangling_1st(std::ranges::reverse, in);
