@@ -25,19 +25,21 @@ int main(int, char**)
     {
         std::ifstream fso("test.dat");
         std::ifstream fs;
-        fs = move(fso);
+        fs = std::move(fso);
         double x = 0;
         fs >> x;
         assert(x == 3.25);
     }
+#ifndef TEST_HAS_NO_WIDE_CHARACTERS
     {
         std::wifstream fso("test.dat");
         std::wifstream fs;
-        fs = move(fso);
+        fs = std::move(fso);
         double x = 0;
         fs >> x;
         assert(x == 3.25);
     }
+#endif
 
   return 0;
 }
