@@ -6,7 +6,7 @@
 //
 //===----------------------------------------------------------------------===//
 
-// UNSUPPORTED: c++03
+// UNSUPPORTED: c++03, c++11, c++14
 // UNSUPPORTED: availability-filesystem-missing
 
 // These tests require locale for non-char paths
@@ -30,8 +30,7 @@
 // template <class InputIterator>
 //   path& concat(InputIterator first, InputIterator last);
 
-
-#include "filesystem_include.h"
+#include <filesystem>
 #include <type_traits>
 #include <string>
 #include <string_view>
@@ -45,7 +44,7 @@
 #include "make_string.h"
 #include "test_iterators.h"
 #include "test_macros.h"
-
+namespace fs = std::filesystem;
 
 struct ConcatOperatorTestcase {
   MultiStringType lhs;
