@@ -12,7 +12,7 @@
 
 // <locale>
 
-// ADDITIONAL_COMPILE_FLAGS: -D_LIBCPP_DISABLE_DEPRECATION_WARNINGS
+// ADDITIONAL_COMPILE_FLAGS: -D_LIBCPP_DISABLE_DEPRECATION_WARNINGS -D_LIBCPP_ENABLE_CXX26_REMOVED_CODECVT
 
 // wstring_convert<Codecvt, Elem, Wide_alloc, Byte_alloc>
 
@@ -21,9 +21,10 @@
 
 // XFAIL: no-wide-characters
 
-#include <locale>
-#include <codecvt>
 #include <cassert>
+#include <codecvt>
+#include <locale>
+#include <type_traits>
 
 #include "test_macros.h"
 
