@@ -35,6 +35,10 @@
 
 int condition_variable_lock_skipped_counter = 0;
 
+TEST_DIAGNOSTIC_PUSH
+// MSVC warning C4583: 'X::cv_': destructor is not implicitly called
+TEST_MSVC_DIAGNOSTIC_IGNORED(4583)
+
 union X {
     X() : cv_() {}
     ~X() {}
@@ -42,6 +46,8 @@ union X {
     unsigned char bytes_[sizeof(std::condition_variable)];
 };
 
+TEST_DIAGNOSTIC_POP
+
 void test()
 {
     constexpr int N = 3;
